"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path = require("path");
const from_dir_1 = require("./from-dir");
const string_builder_1 = require("./string-builder");
function build(rootPath, noChunk) {
    return __awaiter(this, void 0, void 0, function* () {
        const chunkMode = noChunk ? 'eager' : 'lazy';
        const home = path.resolve(rootPath);
        const sandboxes = findSandboxes(home);
        const filePath = path.resolve(__dirname, '../build/app/shared/sandboxes.js');
        const fileContent = buildSandboxFileContents(sandboxes, home, chunkMode);
        // TODO: Remove next release post 3.1.0
        deleteDeprecatedSandboxFileIfNecessary(home);
        return new Promise((resolve, reject) => {
            fs.writeFile(filePath, fileContent, function (err) {
                if (err) {
                    reject(err);
                }
                else {
                    console.log(`Created file: ${filePath}`);
                    resolve(filePath);
                }
            });
        });
    });
}
exports.build = build;
function findSandboxes(home) {
    const sandboxes = [];
    from_dir_1.fromDir(home, /\.sandbox.ts$/, (filename) => {
        let sandboxPath = filename.replace(home, '.').replace(/.ts$/, '').replace(/\\/g, '/');
        const contents = fs.readFileSync(filename, 'utf8');
        const matchSandboxOf = /\s?sandboxOf\s*\(\s*([^)]+?)\s*\)/g.exec(contents);
        if (matchSandboxOf) {
            const typeName = matchSandboxOf[1].split(',')[0].trim();
            const labelText = /label\s*:\s*['"](.+)['"]/g.exec(matchSandboxOf[0]);
            let scenarioMenuItems = [];
            const scenarioRegex = /\.add\s*\(['"](.+)['"]\s*,\s*{/g;
            let scenarioMatches;
            let scenarioIndex = 1;
            while ((scenarioMatches = scenarioRegex.exec(contents)) !== null) {
                scenarioMenuItems.push({ key: scenarioIndex, description: scenarioMatches[1] });
                scenarioIndex++;
            }
            let label = labelText ? labelText[1] : '';
            sandboxes.push({
                key: sandboxPath,
                searchKey: `${typeName}${label}`,
                name: typeName,
                label: label,
                scenarioMenuItems
            });
        }
    });
    return sandboxes;
}
exports.findSandboxes = findSandboxes;
function buildSandboxFileContents(sandboxes, home, chunkMode) {
    const content = new string_builder_1.StringBuilder();
    content.addLine(`function getSandboxMenuItems() {`);
    content.addLine(`return ${JSON.stringify(sandboxes)};`);
    content.addLine(`}`);
    content.addLine(`function getSandbox(path) {`);
    content.addLine(`switch(path) {`);
    sandboxes.forEach(({ key }, i) => {
        let fullPath = path.join(home, key);
        // Normalize slash syntax for Windows/Unix filepaths
        fullPath = slash(fullPath);
        content.addLine(`case '${key}':`);
        content.addLine(`  return import( /* webpackMode: "${chunkMode}" */ '${fullPath}').then(_ => _.default.serialize('${key}'));`);
    });
    content.addLine(`}`);
    content.addLine(`}`);
    content.addLine('export { getSandbox, getSandboxMenuItems };');
    return content.dump();
}
exports.buildSandboxFileContents = buildSandboxFileContents;
// Provided for minor release post sandboxes.ts resolution changes
function deleteDeprecatedSandboxFileIfNecessary(home) {
    const sandboxesFile = path.resolve(home, './sandboxes.ts');
    if (fs.existsSync(sandboxesFile)) {
        fs.unlinkSync(sandboxesFile);
    }
}
// Turns windows URL string ('c:\\etc\\') into URL node expects ('c:/etc/')
// https://github.com/sindresorhus/slash
function slash(input) {
    const isExtendedLengthPath = /^\\\\\?\\/.test(input);
    const hasNonAscii = /[^\u0000-\u0080]+/.test(input);
    if (isExtendedLengthPath || hasNonAscii) {
        return input;
    }
    return input.replace(/\\/g, '/');
}
