"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const net = require("net");
// Legacy import
const detect = require('async/detect');
/**
 * Function that detects the first port not in use in a given range
 * e.g.
 *   findFirstFreePort('127.0.0.1', 8000, 8030, (port) => {
 *       console.log(port)
 *   });
 *
 * @param host - Host to check ports
 * @param start - Starting point for range
 * @param end - Ending point for range
 * @param callback - Callback on result
 */
function findFirstFreePort(host, start, end) {
    return __awaiter(this, void 0, void 0, function* () {
        const ports = [];
        for (let i = start; i < end; i++) {
            ports.push(i);
        }
        const probe = (port, cb) => {
            let calledOnce = false;
            let connected = false;
            const server = net.createServer().listen(port, host);
            const timeoutRef = setTimeout(() => {
                calledOnce = true;
                cb(port, false);
            }, 2000);
            // Active timeout won't require node event loop to remain active
            timeoutRef.unref();
            server.on('listening', () => {
                clearTimeout(timeoutRef);
                if (server)
                    server.close();
                if (!calledOnce) {
                    calledOnce = true;
                    cb(port, true);
                }
            });
            server.on('error', (err) => {
                clearTimeout(timeoutRef);
                let result = true;
                if (err.code === 'EADDRINUSE' || err.code === 'EACCES') {
                    result = false;
                }
                if (!calledOnce) {
                    calledOnce = true;
                    cb(port, result);
                }
            });
        };
        return new Promise(resolve => {
            detect(ports, probe, (port) => {
                resolve(port);
            });
        });
    });
}
exports.findFirstFreePort = findFirstFreePort;
